package model;

// Note!  Nothing from the view package is imported here.
import java.util.Vector;
import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.undo.UndoableEdit;

public class TriangleModel extends Object {
	/* A list of the model's views. */
	private Vector<IViewUpdate> views = new Vector<IViewUpdate>();
	
	// There should be only one instance of the model. 
	private static TriangleModel instance = new TriangleModel();

	// Limit the size of the triangle.
	public static final double MAX_SIDE = 100.0;
	public static final double MAX_HYPO = Math.sqrt(MAX_SIDE * MAX_SIDE
			+ MAX_SIDE * MAX_SIDE);

	private double base = 50.0; // length of the base
	private double height = 50.0; // height of the triangle

	private TriangleModel() {}
	
	/** There should be only one instance of the model.  Use this
	 * method to get it.
	 */
	public static TriangleModel getInstance() {
		return TriangleModel.instance;
	}
	
	/** Set the base to a new value. Must be between 0 and MAX_BASE. */
	public void setBase(double theBase) {
		double tmp = Math.max(1, theBase);
		this.base = Math.min(tmp, MAX_SIDE);
		this.updateAllViews();
	}

	/** Set the height to a new value. Must be between 0 and MAX_HEIGHT. */
	public void setHeight(double theHeight) {
		double tmp = Math.max(1, theHeight);
		this.height = Math.min(tmp, MAX_SIDE);
		this.updateAllViews();
	}

	/** Set both the base and the height to new values. */
	public void setValues(double theBase, double theHeight) {
		double tmp = Math.max(1, theBase);
		this.base = Math.min(tmp, MAX_SIDE);

		tmp = Math.max(1, theHeight);
		this.height = Math.min(tmp, MAX_SIDE);

		this.updateAllViews();
	}

	/** Get the length of this triangle's base. */
	public double getBase() {
		return this.base;
	}

	/** Get this triangle's height. */
	public double getHeight() {
		return this.height;
	}

	/** Get the length of this triangle's hypotenuse. */
	public double getHypotenuse() {
		return Math.sqrt(this.base * this.base + this.height * this.height);
	}
	
	/* Get a command object for setting the triangle's base. */
	public UndoableEdit getSetBaseCommand(final double newBase) {
		return getSetBaseCommand(this.getBase(), newBase);
	}
	
	/* Get a command object for setting the triangle's base. */
	public UndoableEdit getSetBaseCommand(final double oldBase, final double newBase) {
		return new AbstractUndoableEdit() {
			private double oldVal = oldBase;
			private double newVal = newBase;
			
			public void redo () {
				System.err.println("redo base: " + this.newVal);
				super.redo();
				TriangleModel.this.setBase(this.newVal);
			}
			
			public void undo() {
				System.err.println("undo base: " + this.oldVal);
				super.undo();
				TriangleModel.this.setBase(this.oldVal);
			}
		};
	}
	
	/* Get a command object for setting the triangle's height. */
	public UndoableEdit getSetHeightCommand(final double newHeight) {
		return getSetHeightCommand(this.getHeight(), newHeight);
	}
	
	
	/* Get a command object for setting the triangle's height. */
	public UndoableEdit getSetHeightCommand(final double oldHeight, final double newHeight) {
		return new AbstractUndoableEdit() {
			private double oldVal = oldHeight;
			private double newVal = newHeight;
			
			public void redo () {
				super.redo();
				TriangleModel.this.setHeight(this.newVal);
			}
			
			public void undo() {
				super.undo();
				TriangleModel.this.setHeight(this.oldVal);
			}
		};
	}
	
	/* Get a command object for setting the triangle's base and height. */
	public UndoableEdit getSetValuesCommand(final double newBase, final double newHeight) {
		return getSetValuesCommand(this.getBase(),newBase, this.getHeight(), newHeight);
	}
	
	
	/* Get a command object for setting the triangle's height. */
	public UndoableEdit getSetValuesCommand(final double oldBase, final double newBase, final double oldHeight, final double newHeight) {
		return new AbstractUndoableEdit() {
			private double oldBVal = oldBase;
			private double newBVal = newBase;
			private double oldHVal = oldHeight;
			private double newHVal = newHeight;
			
			public void redo () {
				super.redo();
				TriangleModel.this.setValues(this.newBVal, this.newHVal);
			}
			
			public void undo() {
				super.undo();
				TriangleModel.this.setValues(this.oldBVal, this.oldHVal);
			}
		};
	}

	/** Add a new view of this triangle. */
	public void addView(IViewUpdate view) {
		this.views.add(view);
		view.updateView();
	}

	/** Remove a view from this triangle. */
	public void removeView(IViewUpdate view) {
		this.views.remove(view);
	}

	/** Update all the views that are viewing this triangle. */
	private void updateAllViews() {
		for (IViewUpdate view : this.views) {
			view.updateView();
		}
	}
	
}